/*
   Class Name:

      CCamera.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class represents a camera in a 3D scene.
*/


#include"CCamera.h"


CCamera::CCamera()
{
   // Initialize variables...
   xPos, yPos, zPos = 0;
   xView, yView, zView = 0;
   xUp, yUp, zUp = 0;
}


void CCamera::SetCamera(float x, float y, float z,
                        float xv, float yv, float zv,
                        float xu, float yu, float zu)
{
   // Here we set the camera to the values sent in to us.  This is mostly
   // used to set up a default position.
   xPos = x;   yPos = y;   zPos = z;
   xView = xv; yView = yv; zView = zv;
   xUp = xu;   yUp = yu;   zUp = zu;
}


void CCamera::MoveCamera(float direction)
{
   // Moving the camera requires a little more then adding 1 to the z or subracting 1.
   // First we need to get the direction at which we are looking.
   float xLookDirection = 0, yLookDirection = 0, zLookDirection = 0;

   // The look direction is the view (where we are looking) minus the position (where we are).
   xLookDirection = xView - xPos;
	yLookDirection = yView - yPos;
	zLookDirection = zView - zPos;

   // Normalize the direction.
   float dp = 1 /(float)sqrt(xLookDirection * xLookDirection + yLookDirection * yLookDirection +
                             zLookDirection * zLookDirection);
   xLookDirection *= dp;
   yLookDirection *= dp;
   zLookDirection *= dp;

   // To correctly move the camera foward and backwards we simply add the result of the
   // look direction * the speed of our camera to get the new position.  This will move the
   // camera but what about the view?  If the view was set to look at the point (0, 0, 0)
   // then when we move the camera forward or backward the camera's view will stay
   // focused on that one point.  We don't want that so we also effect the view data.

   // Move the camera on the X and Z axis.
	xPos += xLookDirection * direction;
	zPos += zLookDirection * direction;

   // Move the view along with the position
	xView += xLookDirection * direction;
	zView += zLookDirection * direction;
}


void CCamera::RotateCamera(float AngleDir, float xSpeed, float ySpeed, float zSpeed)
{
	float xNewLookDirection = 0, yNewLookDirection = 0, zNewLookDirection = 0;
	float xLookDirection = 0, yLookDirection = 0, zLookDirection = 0;
   float CosineAngle = 0, SineAngle = 0;

   // First we will need to calculate the cos and sine of our angle.
   CosineAngle = (float)cos(AngleDir);
   SineAngle = (float)sin(AngleDir);

	// Next get the look direction just like in the move camera function.
   xLookDirection = xView - xPos;
	yLookDirection = yView - yPos;
	zLookDirection = zView - zPos;

   // Normalize the direction.
   float l = 1 / (float)sqrt(xLookDirection * xLookDirection +
                             yLookDirection * yLookDirection +
                             zLookDirection * zLookDirection);
   xLookDirection *= l;
   yLookDirection *= l;
   zLookDirection *= l;

	// Calculate the new X position.
	xNewLookDirection = (CosineAngle + (1 - CosineAngle) * xSpeed) * xLookDirection;
	xNewLookDirection += ((1 - CosineAngle) * xSpeed * ySpeed - zSpeed * SineAngle)* yLookDirection;
	xNewLookDirection += ((1 - CosineAngle) * xSpeed * zSpeed + ySpeed * SineAngle) * zLookDirection;

	// Calculate the new Y position.
	yNewLookDirection = ((1 - CosineAngle) * xSpeed * ySpeed + zSpeed * SineAngle) * xLookDirection;
	yNewLookDirection += (CosineAngle + (1 - CosineAngle) * ySpeed) * yLookDirection;
	yNewLookDirection += ((1 - CosineAngle) * ySpeed * zSpeed - xSpeed * SineAngle) * zLookDirection;

	// Calculate the new Z position.
	zNewLookDirection = ((1 - CosineAngle) * xSpeed * zSpeed - ySpeed * SineAngle) * xLookDirection;
	zNewLookDirection += ((1 - CosineAngle) * ySpeed * zSpeed + xSpeed * SineAngle) * yLookDirection;
	zNewLookDirection += (CosineAngle + (1 - CosineAngle) * zSpeed) * zLookDirection;

	// Last we add the new rotations to the old view to correctly rotate the camera.
	xView = xPos + xNewLookDirection;
	yView = yPos + yNewLookDirection;
	zView = zPos + zNewLookDirection;
}


// Copyright October 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com